@@ -2,40 +2,37 @@
 #include <Rmath.h>
 #include <R_ext/Lapack.h>
 
-/* -------------------- some useful macros ------------------------------ */
-
-#define NROWS(x) INTEGER(getAttrib(x, R_DimSymbol))[0]
-#define NCOLS(x) INTEGER(getAttrib(x, R_DimSymbol))[1]
-
 /* -------------------- function declarations --------------------------- */
 
 static void c_svd(double *A, double *U, double *D, double *V, int *nrows,
     int *ncols, int *mindim);
+double c_det(double *matrix, int *rows);
 
 /* -------------------- R level interfaces to LAPACK -------------------- */
 
 /* Singular Value Decomposition (SVD). */
 SEXP r_svd(SEXP matrix) {
 
-  SEXP U, D, Vt, result, elnames;
-  int nrows = 0, ncols = 0, mindim = 0;
+SEXP U, D, Vt, result, elnames;
+int nr = 0, nc = 0, mindim = 0;
+short int duplicated = 0;
 
   /* compute the dimensions of the SVD components. */
-  nrows = NROWS(matrix);
-  ncols = NCOLS(matrix);
-  mindim = imin2(nrows, ncols);
+  nr = nrows(matrix);
+  nc = ncols(matrix);
+  mindim = imin2(nr, nc);
 
   /* allocate the U, D and t(V) matrices. */
-  PROTECT(U = allocMatrix(REALSXP, nrows, mindim));
-  PROTECT(Vt = allocMatrix(REALSXP, mindim, ncols));
+  PROTECT(U = allocMatrix(REALSXP, nr, mindim));
+  PROTECT(Vt = allocMatrix(REALSXP, mindim, nc));
   PROTECT(D = allocVector(REALSXP, mindim));
 
   /* duplicate the matrix so the original one is not overwritten. */
-  if (NAMED(matrix) > 0)
+  if ((duplicated = NAMED(matrix)) > 0)
     PROTECT(matrix = duplicate(matrix));
 
   /* call Lapack to do the grunt work. */
-  c_svd(REAL(matrix), REAL(U), REAL(D), REAL(Vt), &nrows, &ncols, &mindim);
+  c_svd(REAL(matrix), REAL(U), REAL(D), REAL(Vt), &nr, &nc, &mindim);
 
   /* build the return value. */
   PROTECT(result = allocVector(VECSXP, 3));
@@ -48,7 +45,7 @@ SEXP r_svd(SEXP matrix) {
   SET_VECTOR_ELT(result, 1, U);
   SET_VECTOR_ELT(result, 2, Vt);
 
-  if (NAMED(matrix) > 0)
+  if (duplicated > 0)
     UNPROTECT(6);
   else
     UNPROTECT(5);
@@ -57,6 +54,35 @@ SEXP r_svd(SEXP matrix) {
 
 }/*R_SVD*/
 
+/* Determinant. */
+SEXP r_det(SEXP matrix, int scale) {
+
+int i = 0, nr = nrows(matrix);
+short int duplicated = 0;
+SEXP result;
+
+  PROTECT(result = allocVector(REALSXP, 1));
+
+  /* duplicate the matrix so the original one is not overwritten. */
+  if ((duplicated = NAMED(matrix)) > 0)
+    PROTECT(matrix = duplicate(matrix));
+
+  /* rescale the determinant by multiplying the matrix. */
+  for (i = 0; i < nr * nr; i++)
+    REAL(matrix)[i] *= scale;
+
+  /* call Lapack to do the grunt work. */
+  NUM(result) = c_det(REAL(matrix), &nr);
+
+  if (duplicated > 0)
+    UNPROTECT(2);
+  else
+    UNPROTECT(1);
+
+  return result;
+
+}/*R_DET*/
+
 /* -------------------- C level interfaces to LAPACK -------------------- */
 
 /* C-level wrapper around the dgesvd() F77 routine. Note that the input
@@ -65,9 +91,9 @@ SEXP r_svd(SEXP matrix) {
 static void c_svd(double *A, double *U, double *D, double *V, int *nrows,
     int *ncols, int *mindim) {
 
-  int err, lwork = -1;
-  char jobu, jobvt;
-  double work1, *work;
+int err = 0, lwork = -1;
+char jobu, jobvt;
+double work1 = 0, *work = NULL;
 
   if (*nrows < *ncols) {
 
@@ -102,3 +128,46 @@ static void c_svd(double *A, double *U, double *D, double *V, int *nrows,
 
 }/*C_SVD*/
 
+/* C-level function to compute the determinant of a real-valued square matrix,
+ * modeled after the moddet_ge_real() function in Lapack.c. */
+double c_det(double *matrix, int *rows) {
+
+int sign = 1, i = 0, info = 0, *jpvt = NULL;
+double det = 1;
+
+  jpvt = (int *) Calloc(*rows, int);
+
+  /* comute the A = L*U decomposition. */
+  F77_CALL(dgetrf)(rows, rows, matrix, rows, jpvt, &info);
+
+  if (info < 0) {
+
+    error("an error (%d) occurred in the call to dgesvd().\n", info);
+
+  }/*THEN*/
+  else if (info > 0) {
+
+    /* the matrix is singular, so the determinant is zero. */
+    det = 0;
+
+  }/*THEN*/
+  else {
+
+    /* the matrix is full rank, compute the determinant. */
+    for (i = 0; i < *rows; i++) {
+
+      if (jpvt[i] != (i + 1))
+        sign = -sign;
+
+      det *= matrix[CMC(i, i, *rows)];
+
+    }/*FOR*/
+
+  }/*ELSE*/
+
+  Free(jpvt);
+
+  return sign * det;
+
+}/*C_DET*/
+
