@@ -1,30 +1,30 @@
 #include "common.h"
 
-#define NODE(i) CHAR(STRING_ELT(nodes, i))
-
 SEXP has_pdag_path(SEXP from, SEXP to, SEXP amat, SEXP nrows, SEXP nodes,
     SEXP underlying, SEXP exclude_direct, SEXP debug) {
 
-  int start = INT(from) - 1;
-  int stop = INT(to) - 1;
-  int n = INT(nrows);
-  int *a = INTEGER(amat);
+int start = INT(from) - 1, stop = INT(to) - 1, n = INT(nrows);
+int debuglevel = LOGICAL(debug)[0], notdirect = LOGICAL(exclude_direct)[0],
+      ugraph = LOGICAL(underlying)[0], *a = INTEGER(amat);
+SEXP result;
 
-  return c_has_path(start, stop, a, n, nodes, underlying, exclude_direct, debug);
+  PROTECT(result = allocVector(LGLSXP, 1));
+  LOGICAL(result)[0] =
+    c_has_path(start, stop, a, n, nodes, ugraph, notdirect, debuglevel);
+
+  UNPROTECT(1);
+  return result;
 
 }/*HAS_DAG_PATH*/
 
-SEXP c_has_path(int start, int stop, int *amat, int n, SEXP nodes,
-    SEXP underlying, SEXP exclude_direct, SEXP debug) {
+int c_has_path(int start, int stop, int *amat, int n, SEXP nodes,
+    int ugraph, int notdirect, int debuglevel) {
 
-  int *counter, *path;
-  int path_pos = 0, cur = start;
-  int i = 0, a1 = 0, a2 = 0;
-
-  SEXP res;
+int *counter = NULL, *path = NULL;
+int i = 0, a1 = 0, a2 = 0, path_pos = 0, cur = start;
 
   /* remove any arc between start and stop if asked to. */
-  if (isTRUE(exclude_direct)) {
+  if (notdirect) {
 
     a1 = amat[CMC(start, stop, n)];
     a2 = amat[CMC(stop, start, n)];
@@ -37,14 +37,10 @@ SEXP c_has_path(int start, int stop, int *amat, int n, SEXP nodes,
   /* initialize the path array. */
   path = alloc1dcont(n);
 
-  /* allocate the result. */
-  PROTECT(res = allocVector(LGLSXP, 1));
-  LOGICAL(res)[0] = TRUE;
-
   /* iterate until the other node is found. */
   while (cur != stop) {
 
-    if (isTRUE(debug)) {
+    if (debuglevel > 0) {
 
       Rprintf("* currently at '%s'.\n", NODE(cur));
       Rprintf("  > current path is:\n");
@@ -59,7 +55,7 @@ there:
     /* find the next child of the 'cur' node. */
     for (i = 0; (i < n) && (counter[cur] < n); i++) {
 
-      if (!isTRUE(underlying)) {
+      if (!ugraph) {
 
         if (amat[CMC(cur, counter[cur], n)] != 0)
           break;
@@ -87,22 +83,19 @@ there:
        * and the 'stop' node was not found; return FALSE. */
       if  (path_pos == 0) {
 
-        LOGICAL(res)[0] = FALSE;
-        UNPROTECT(1);
-
         /* remove any arc between start and stop if asked to. */
-        if (isTRUE(exclude_direct)) {
+        if (notdirect) {
 
           amat[CMC(start, stop, n)] = a1;
           amat[CMC(stop, start, n)] = a2;
 
         }/*THEN*/
 
-        return res;
+        return FALSE;
 
       }/*THEN*/
 
-      if (isTRUE(debug))
+      if (debuglevel > 0)
         Rprintf("  > node '%s' has no more children, going back to '%s'.\n",
           NODE(cur), NODE(path[path_pos - 1]));
 
@@ -125,7 +118,7 @@ there:
 
         if ((counter[cur] - 1) == path[i]) {
 
-          if (isTRUE(debug))
+          if (debuglevel > 0)
             Rprintf("  @ node '%s' already visited, skipping.\n", NODE(path[i]));
 
           goto there;
@@ -139,36 +132,38 @@ there:
       /* the current node is now the children we have just found. */
       cur = counter[cur] - 1;
 
-      if (isTRUE(debug))
+      if (debuglevel > 0)
         Rprintf("  > jumping to '%s'.\n", NODE(cur));
 
     }/*ELSE*/
 
   }/*WHILE*/
 
-  /* node 'stop' has been found, return TRUE. */
-  UNPROTECT(1);
 
   /* remove any arc between start and stop if asked to. */
-  if (isTRUE(exclude_direct)) {
+  if (notdirect) {
 
     amat[CMC(start, stop, n)] = a1;
     amat[CMC(stop, start, n)] = a2;
 
   }/*THEN*/
 
-  return res;
+  /* node 'stop' has been found, return TRUE. */
+  return TRUE;
 
 }/*C_HAS_DAG_PATH*/
 
 SEXP how_many_cycles(SEXP from, SEXP to, SEXP amat, SEXP nrows, SEXP nodes, SEXP debug) {
 
-  int start = INT(from) - 1, stop = INT(to) - 1, n = INT(nrows);
-  int *counter, *path, *a;
-  int path_pos = 0, cur = start, cycle_counter = 0;
-  int i = 0, a1 = 0, a2 = 0;
+int start = INT(from) - 1, stop = INT(to) - 1, n = INT(nrows);
+int *counter = NULL, *path = NULL, *a = NULL;
+int path_pos = 0, cur = start, cycle_counter = 0;
+int i = 0, a1 = 0, a2 = 0;
+int *debuglevel = NULL;
+SEXP res;
 
-  SEXP res;
+  /* dereference the debug parameter. */
+  debuglevel = LOGICAL(debug);
 
   /* initialize the position counters for the rows of the adjacency matrix. */
   counter = alloc1dcont(n);
@@ -179,7 +174,7 @@ SEXP how_many_cycles(SEXP from, SEXP to, SEXP amat, SEXP nrows, SEXP nodes, SEXP
   PROTECT(res = allocVector(INTSXP, 1));
   INT(res) = 0;
 
-  /* map the adjacency matri. */
+  /* map the adjacency matrix. */
   a = INTEGER(amat);
   /* back up and remove any arc between "start" and "stop".  */
   a1 = a[CMC(start, stop, n)];
@@ -188,7 +183,7 @@ SEXP how_many_cycles(SEXP from, SEXP to, SEXP amat, SEXP nrows, SEXP nodes, SEXP
 
   while (1) {
 
-    if (isTRUE(debug))
+    if (*debuglevel > 0)
       Rprintf("* currently at '%s'.\n", NODE(cur));
 
 there:
@@ -207,7 +202,7 @@ there:
 
       cycle_counter++;
 
-      if (isTRUE(debug)) {
+      if (*debuglevel > 0) {
 
         Rprintf("  @ found node '%s' ! cycle counter is now %d.\n",
           NODE(stop), cycle_counter);
@@ -245,7 +240,7 @@ there:
 
       }/*THEN*/
 
-      if (isTRUE(debug)) {
+      if (*debuglevel > 0) {
 
         Rprintf("  > node '%s' has no more children, going back to '%s'.\n",
           NODE(cur), NODE(path[path_pos - 1]));
@@ -269,7 +264,7 @@ there:
 
         if ((counter[cur] - 1) == path[i]) {
 
-          if (isTRUE(debug)) {
+          if (*debuglevel > 0) {
 
             Rprintf("  @ node '%s' already visited, skipping.\n", NODE(path[i]));
 
@@ -286,7 +281,7 @@ there:
       /* the current node is now the children we have just found. */
       cur = counter[cur] - 1;
 
-      if (isTRUE(debug)) {
+      if (*debuglevel > 0) {
 
         Rprintf("  > jumping to '%s'.\n", NODE(cur));
 
